package com.jiuqi.grid;

import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.io.Serializable;

public class Matrix implements Serializable
{
    private int xCount;
    private int yCount;
    private List list;
    
    public Matrix() {
        this.list = new ArrayList();
    }
    
    public int getXCount() {
        return this.xCount;
    }
    
    public int getYCount() {
        return this.yCount;
    }
    
    public void setSize(final int xSize, final int ySize) {
        if (xSize < 0 || ySize < 0) {
            return;
        }
        int c = 0;
        if (this.xCount != xSize) {
            for (int i = this.xCount; i < xSize; ++i) {
                this.list.add(new ArrayList());
            }
            int i;
            for (c = (i = this.list.size()); i > xSize; --i) {
                this.list.remove(i - 1);
            }
            this.xCount = xSize;
        }
        if (this.yCount < ySize) {
            for (int i = 0; i < this.xCount; ++i) {
                final List data = this.list.get(i);
                if (data != null) {
                    for (int j = this.yCount; j < ySize; ++j) {
                        data.add(null);
                    }
                }
            }
            this.yCount = ySize;
        }
        if (this.yCount > ySize) {
            for (int i = 0; i < this.xCount; ++i) {
                final List data = this.list.get(i);
                if (data != null) {
                    int j;
                    for (c = (j = data.size()); j > ySize; --j) {
                        data.remove(j - 1);
                    }
                }
            }
            this.yCount = ySize;
        }
    }
    
    public void resetSize(final int xSize, final int ySize) {
        this.setSize(xSize, ySize);
        for (int x = 0; x < this.getXCount(); ++x) {
            for (int y = 0; y < this.getYCount(); ++y) {
                this.setElement(x, y, null);
            }
        }
    }
    
    public Object getElement(final int x, final int y) {
        if (x < 0 || y < 0 || x >= this.xCount || y >= this.yCount) {
            return null;
        }
        final List data = this.list.get(x);
        if (data != null && y < data.size()) {
            return data.get(y);
        }
        return null;
    }
    
    public void setElement(final int x, final int y, final Object obj) {
        if (x >= 0 && y >= 0 && x < this.xCount && y < this.yCount) {
            List data = this.list.get(x);
            if (data == null && obj != null) {
                data = new ArrayList();
                this.list.set(x, data);
            }
            if (data != null) {
                for (int i = data.size(); i < this.yCount; ++i) {
                    data.add(null);
                }
                data.set(y, obj);
            }
        }
    }
    
    public boolean hasElement(final Object o) {
        for (int i = 0; i < this.list.size(); ++i) {
            final List data = this.list.get(i);
            if (data != null && data.indexOf(o) >= 0) {
                return true;
            }
        }
        return false;
    }
    
    public void xDelete(final int x) {
        this.list.remove(x);
        --this.xCount;
    }
    
    public void xDelete(final int x, final int count) {
        for (int i = 0; i < count; ++i) {
            this.xDelete(x);
        }
    }
    
    public void yDelete(final int y) {
        for (int i = 0; i < this.list.size(); ++i) {
            final List data = this.list.get(i);
            if (data != null && y < data.size()) {
                data.remove(y);
            }
        }
        --this.yCount;
    }
    
    public void yDelete(final int y, final int count) {
        for (int i = 0; i < count; ++i) {
            this.yDelete(y);
        }
    }
    
    public void xInsert(final int x, final int count) {
        for (int i = 0; i < count; ++i) {
            this.list.add(x, null);
        }
        this.xCount += count;
    }
    
    public void yInsert(final int y, final int count) {
        if (y < 0 || y > this.yCount) {
            return;
        }
        for (int i = 0; i < this.list.size(); ++i) {
            final List data = this.list.get(i);
            if (data != null && y <= data.size()) {
                for (int k = 0; k < count; ++k) {
                    data.add(y, null);
                }
            }
        }
        this.yCount += count;
    }
    
    public void xExchange(final int x1, final int x2) {
        if (x1 < 0 || x2 < 0 || x1 >= this.xCount || x2 >= this.xCount) {
            return;
        }
        final Object data = this.list.get(x1);
        this.list.set(x1, this.list.get(x2));
        this.list.set(x2, data);
    }
    
    public void yExchange(final int y1, final int y2) {
        if (y1 < 0 || y2 < 0 || y1 >= this.yCount || y2 >= this.yCount || y1 == y2) {
            return;
        }
        for (int i = 0; i < this.list.size(); ++i) {
            final List data = this.list.get(i);
            if (data != null) {
                if (y1 < data.size() || y2 < data.size()) {
                    for (int k = data.size(); k < this.yCount; ++k) {
                        data.add(null);
                    }
                    final Object o = data.get(y1);
                    data.set(y1, data.get(y2));
                    data.set(y2, o);
                }
            }
        }
    }
    
    public void xCopy(final int fromIndex, final int toIndex) {
        final List data = this.list.get(fromIndex);
        if (data == null) {
            this.list.set(toIndex, null);
        }
        else {
            final List newData = new ArrayList();
            newData.addAll(data);
            this.list.set(toIndex, newData);
        }
    }
    
    public void yCopy(final int fromIndex, final int toIndex) {
        for (int i = 0; i < this.list.size(); ++i) {
            final List data = this.list.get(i);
            if (data != null) {
                if (fromIndex < data.size() || toIndex < data.size()) {
                    for (int k = data.size(); k < this.yCount; ++k) {
                        data.add(null);
                    }
                    data.set(toIndex, data.get(fromIndex));
                }
            }
        }
    }
}
